/* HelloCube.c

	this is a source code example of a program using "3D Game Machine"...
	(this demo needs 800 Kb to run)

	3D Game Machine Copyright (c) Virtually Unlimited, SNC 1995 all right reserved
*/

// system includes
#include "Quickdraw.h"
#include "Palettes.h"
#include "QDOffscreen.h"

// 3DGM includes
#include "DebugU.h"
#include "t3D types.h"
#include "t3D.h"
#include "t3D_U.h"
#include "t3D_Conv.h"
#include "t3D_Math.h"
#include "t3D_MacOS.h"
#include "t3D_OSUtils.h"
#include "t3D_IO.h"
#include "t3D_Modeler.h"

// function prototypes

void	InitMac(void);
void	SetScreenDepth(short NewDepth);
void	Quit(void);
Ptr	GetPicturePtr(short PictID);

#define		WorkBuffSize	150*1024L
#define		PrimBuffSize	1024L

#define		PicSize		128L

// global variables

GDHandle	DaGDevice;
Ptr		DaPicPtr;

short	SavedDepth,SavedColor;

// White light coefficients
static	RGBARec	WhiteAmbient[]		= {0xF000,0xF000,0xF000,0} ;
static	RGBARec	WhiteDiffuse[]		= {0xF000,0xF000,0xF000,0} ;
static	RGBARec	WhiteSpecular[]		= {0,0,0,0} ;

// sample material coefficients
static	RGBARec	Mat1Emission[]		= {0,0,0,0} ;
static	RGBARec	Mat1Ambient[]		= {-1,0xC000,0xC000,0} ;
static	RGBARec	Mat1Diffuse[]		= {-1,0xC000,0xC000,0} ;
static	RGBARec	Mat1Specular[]		= {0x2000,0x2000,0x2000,0} ;

static	RGBARec	Mat2Emission[]		= {0,0,0,0} ;
static	RGBARec	Mat2Ambient[]		= {-1,0x2000,0x2000,0} ;
static	RGBARec	Mat2Diffuse[]		= {-1,0x2000,0x2000,0} ;
static	RGBARec	Mat2Specular[]		= {-1,-1,-1,0} ;

static	short	CubeDots[]={
	
	#define		CubeSize	1000
	
	-CubeSize,-CubeSize,-CubeSize,
	-CubeSize,CubeSize,-CubeSize,
	CubeSize,CubeSize,-CubeSize,
	CubeSize,-CubeSize,-CubeSize,
	
	-CubeSize,-CubeSize,CubeSize,
	-CubeSize,CubeSize,CubeSize,
	CubeSize,CubeSize,CubeSize,
	CubeSize,-CubeSize,CubeSize,
	
	EOD
	
	} ;

#define	MaterialID	1
#define	TextureID	1

static	short	CubePrimitives[]={
	
	TexPoly,TextureID,MaterialID,0,0,0,0,0,4,7,0,0,3,7,4,EOS,
	TexPoly,TextureID,MaterialID,0,0,0,0,0,7,6,3,3,2,6,7,EOS,
	TexPoly,TextureID,MaterialID,0,0,0,0,0,5,4,1,4,5,1,0,EOS,
	TexPoly,TextureID,MaterialID,0,0,0,0,0,5,6,4,7,6,5,4,EOS,
	TexPoly,TextureID,MaterialID,0,0,0,0,0,6,5,2,2,1,5,6,EOS,
	TexPoly,TextureID,MaterialID,0,0,0,0,0,1,2,0,0,1,2,3,EOS,
	
	EOD
	
	} ;
	
static	short	TetraDots[]={
	
	-200,-100,0,
	200,-100,0,
	0,200,0,
	0,0,200,
	
	EOD
	
	} ;

static	short	TetraPrims[]={
	
	Poly,2,0,2,1,0,EOS,		// bottom
	Poly,2,0,1,2,3,EOS,		// right
	Poly,2,0,3,2,0,EOS,		// left
	Poly,2,0,0,1,3,EOS,		// front
	
	EOD
	
	} ;
	
void	InitMac(void)

{	MaxApplZone();
	MoreMasters();
	MoreMasters();
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	InitDialogs(0L);
	DaGDevice=GetGDevice();
}

//---- Set Screen Depth ------------------------------

 void	SetScreenDepth(short NewDepth)

{	short	err;
	
	SavedDepth=(*(*DaGDevice)->gdPMap)->pixelSize;
	SavedColor=(*DaGDevice)->gdFlags;
	
	err=HasDepth(DaGDevice,NewDepth,1,1);
	if (!err) AlertMessage("\pDevice doesnt support 8 bit depth");
	else
	{	err=SetDepth(DaGDevice,8,1,1);
		if (err) AlertMessage("\pSetDepth error");
	}
}

//---- Quit ------------------------------------------

void	Quit(void)

{	short	err;
	
	DisposePtr(DaPicPtr);				// release Picture memory
	
	FlushEvents(everyEvent,0);
	
	// restore saved screen depth
	
	err=SetDepth(DaGDevice,SavedDepth,1,SavedColor);
	if (err) AlertMessage("\pcouldnt reset Depth");
	
	// leave 3DGM's 3D toolbox
	
	err=Exitt3D();
	if (err) AlertMessage("\pcouldnt Exit t3D");
	
	ExitToShell();
}

//---- Get Picture Ptr ----------------------------------

Ptr	GetPicturePtr(short PictID)

{	Rect		picRect;
	PixMapHandle	DaPixMapHdl;
	GWorldPtr	DaGWorld;
	PicHandle	DaPicHdl;
	Ptr		OrgPtr,OrgPtr0,DestPtr,DestPtr0;
	long		orgRow;
	short		i,err;
	
	// load picture in resource
	
	DaPicHdl=GetPicture(PictID);
	if (DaPicHdl==nil)	WayOut("\pPicture not found");
	HLock((Handle)DaPicHdl);
	
	// make an offscreen buffer to draw picture
	
	SetRect(&picRect,0,0,PicSize,PicSize);
	err=NewGWorld(&DaGWorld,8,&picRect,NULL,NULL,0);
	if (err)		WayOut("\pNewGWorld error");
	if (DaGWorld==NULL)	WayOut("\pOut of Memory");
	DaPixMapHdl=GetGWorldPixMap(DaGWorld);
	LockPixels(DaPixMapHdl);                                                                // pouvoir l'utiliser.
	SetGWorld(DaGWorld,0L);
	DrawPicture(DaPicHdl,&picRect);
	
	orgRow=((*DaPixMapHdl)->rowBytes)&0x1FFF;
	OrgPtr0=GetPixBaseAddr(DaPixMapHdl);
	
	// allocate space for depacked picture
	
	DaPicPtr=NewPtr(PicSize*256L);
	if (DaPicPtr==nil)	WayOut("\pOut of Memory");
	
	DestPtr0=DaPicPtr;
	
	// copy picture to get a RowBytes of 256 (always) for texture
	
	for (i=0;i<PicSize;i++)
	{	short	j;
		
		OrgPtr=OrgPtr0;
		DestPtr=DestPtr0;
		for (j=0;j<PicSize;j++)	*DestPtr++=*OrgPtr++;
		
		OrgPtr0+=orgRow;
		DestPtr0+=256;
	}
	
	// release resource and offscreen buffer
	
	UnlockPixels(GetGWorldPixMap(DaGWorld));
	DisposeGWorld(DaGWorld);
	HUnlock((Handle)DaPicHdl);
	ReleaseResource((Handle)DaPicHdl);
	
	return	DaPicPtr;	// returns pointer to picture
}

//---- MAIN CODE -----------------------------------------

main()

{	WorkStructPtr	DaWorkBuffer;			// work buffers
	long		*DaPrimBuffer;
	
	ScreenPort	DaScreenPort;
	ViewPort	DaViewPort;
	WorldRec	DaWorld;
	ShapeRec	TextureCube,Tetrahedron;
	LightSource	WhiteLight;
	
	ScreenParamRec	DaScreen;
	
	Handle		PltHdl;
	Ptr		TexPtr;
	short		err,DaTextureID,DaMaterial1ID,DaMaterial2ID;
	
	InitMac();
	HideCursor();
	SetScreenDepth(8);
	
	InitDebugU();		// init 3DGM Debugger utilities
	SetQuitProc(Quit);	// sets Quit procedure in case of hard(er) crash
	
// Init 3DGM's 3D toolbox
	
	err=Initt3D();
	if (err)	WayOut("\pInitt3D error");
	
	// allocate memory for ViewPort's work buffers
	
	DaWorkBuffer=(WorkStruct *)NewPtr(WorkBuffSize);
	if (DaWorkBuffer==nil)	WayOut("\pOut of Memory");
	
	DaPrimBuffer=(long *)NewPtr(PrimBuffSize);
	if (DaPrimBuffer==nil)		WayOut("\pOut of Memory");
	
// SCREEN PORT
	
	err=InitScreenPort(&DaScreenPort);
	if (err)	WayOut("\pNewScreenPort error");
	
	DaScreenPort.stereo=false;		// no stereo vision
	DaScreenPort.contrast=100;		// contrast at 100%
	DaScreenPort.luminosities=32;		// 32 different luminosities (for texture shading)
	
	err=NewScreenPort(&DaScreenPort);
	if (err)	WayOut("\pInitScreenPort error");
	
	// (Load system palette)
	
	PltHdl=GetResource('clut',999);
	if (PltHdl==nil)	WayOut("\pResource not found");
	HLock(PltHdl);
	SetScreenPortPalette(&DaScreenPort,*PltHdl+8);
	HUnlock(PltHdl);
	ReleaseResource(PltHdl);
	
	// calculate luminosities from current screen palette,
	// you would normally use your own palette to get a
	// nicer shading...
	err=MakeLuminosities(&DaScreenPort,LuminositiesNbr);
	if (err)	WayOut("\pMake Luminosities error");
	
// WORLD
	
	err=InitWorld(&DaWorld);
	if (err)	WayOut("\pInitWorld error");
	
	DaWorld.active=true;
	DaWorld.globalAmbientPercentage=20;			// 20 % global ambient light
	DaWorld.globalAmbientLight.R=65535;			// ambient color is white
	DaWorld.globalAmbientLight.G=65535;
	DaWorld.globalAmbientLight.B=65535;
	
	err=NewWorld(&DaWorld);
	if (err)	WayOut("\pNewWorld error");
	
// VIEW PORT
	
	err=InitViewPort(&DaViewPort);
	if (err)	WayOut("\pInitViewPort error");
	
	DaViewPort.active=true;
	DaViewPort.graphicMode=filled;			// filled 3D
	DaViewPort.world=&DaWorld;			// install World
	DaViewPort.colorMode=bestFitColor;		// color fitting algorithm is "best fit"
	
	// get direct pointer to screen memory (a good boy wouldn't do this normally)
	
	GetScreenParams(&DaScreen);
	DaViewPort.imageBuffer=DaScreen.bitmap+(DaScreen.row*50L)+50L;
	DaViewPort.width=400;
	DaViewPort.height=300;
	DaViewPort.row=DaScreen.row;
	
	DaViewPort.workBuffer=DaWorkBuffer;		// install pointer to work memory buffer
	DaViewPort.bucketPercentage=30;			// vector memory gets 30% of work buffer
	DaViewPort.primitiveList=DaPrimBuffer;		// install pointer to primitives memory buffer
	DaViewPort.screenPort=&DaScreenPort;		// pointer to current screen
	
	// sets position and orientation of observer/camera
	
	DaViewPort.pos.x=0;
	DaViewPort.pos.y=-6000;
	DaViewPort.pos.z=0;
	DaViewPort.pos.alpha=0;
	DaViewPort.pos.beta=0;
	DaViewPort.pos.teta=0;
	DaViewPort.hCenter=DaViewPort.width/2;
	DaViewPort.vCenter=DaViewPort.height/2;
	
	DaViewPort.zoom=500;			// sets camera's opening angle
	DaViewPort.hither=100;			// sets front clipping plane (called "hither")
	DaViewPort.bfCulling=true;		// back-face culling is turned on
	DaViewPort.lightMode=flatShaded;	// flat-shaded view port
	DaViewPort.parallelEye=false;		// does not set observer at infinity for light calculations
	
	err=NewViewPort(&DaViewPort);
	if (err)	WayOut("\pNewViewPort error");
	
	ClearViewPort(&DaViewPort);		// erase the view port
	
// MATERIAL
	
	DaMaterial1ID=NewMaterialID(1);		// ask for material in slot 1
	if (DaMaterial1ID==0)	WayOut("\pNewMaterialID Error");
	
	// define this material's coefficients
	err=DefineMaterial(DaMaterial1ID,Mat1Emission,Mat1Ambient,Mat1Diffuse,Mat1Specular,0);
	if (err)	WayOut("\pDefineMaterial Error");
	
	DaMaterial2ID=NewMaterialID(2);		// ask for material in slot 2
	if (DaMaterial2ID==0)	WayOut("\pNewMaterialID Error");
	
	// define this material's coefficients
	err=DefineMaterial(DaMaterial2ID,Mat2Emission,Mat2Ambient,Mat2Diffuse,Mat2Specular,0);
	if (err)	WayOut("\pDefineMaterial Error");
	
// TEXTURE
	
	DaTextureID=NewTextureID(-1);		// ask for any free texture slot
	if (DaTextureID==0)	WayOut("\pNewTextureID Error");
	
	TexPtr=GetPicturePtr(128);		// gets pointer to picture resource 128 (check subroutine)
	
	err=DefineTexture(DaTextureID,PicSize,PicSize,TexPtr,nil);	// define texture (from loaded pict)
	if (err)	WayOut("\pDefineTexture Error");
	
// LIGHT SOURCE
	
	InitLightSource(&WhiteLight);
	
	WhiteLight.active=true;			// turn light source on
	WhiteLight.type=positional;		// lightsource type is positional
	
	// lightsource position
	WhiteLight.pos.x=-50000;
	WhiteLight.pos.y=-16000;
	WhiteLight.pos.z=30000;
	
	// lightsource coefficients
	WhiteLight.ambient=*WhiteAmbient;
	WhiteLight.diffuse=*WhiteDiffuse;
	WhiteLight.specular=*WhiteSpecular;
	
	NewLightSource(&WhiteLight);
	AttachLightSource(&DaWorld,&WhiteLight);	// attach light source to world
	
	
// TEXTURE CUBE shape
	
	err=InitShape(&TextureCube);
	if (err)	WayOut("\pInitShape error");
	
	TextureCube.type=filledShape;
	TextureCube.active=true;
	TextureCube.dotsPtr=CubeDots;			// install pointer to shape dots
	TextureCube.dotType=shortDot;
	TextureCube.primsPtr=CubePrimitives;		// install pointer to shape primitives
	TextureCube.bfCulling=true;			// turn back-face culling for shape on
	
	// shape position and orientation
	TextureCube.pos.x=0;
	TextureCube.pos.y=0;
	TextureCube.pos.z=0;
	TextureCube.pos.alpha=0;
	TextureCube.pos.beta=0;
	TextureCube.pos.teta=0;
	
	err=NewShape(&TextureCube);
	if (err)	WayOut("\pNewShape error");
	
	err=AttachShape(&DaWorld,&TextureCube);		// atach shape to world
	if (err)	WayOut("\pAttachShape error");
	
// TETRAHEDRON shape
	
	err=InitShape(&Tetrahedron);
	if (err)	WayOut("\pInitShape error");
	
	Tetrahedron.type=filledShape;			// same as with shape 1
	Tetrahedron.active=true;
	Tetrahedron.dotsPtr=TetraDots;
	Tetrahedron.dotType=shortDot;
	Tetrahedron.primsPtr=TetraPrims;
	Tetrahedron.bfCulling=true;
	
	// set Tetrahedron position and orientation
	Tetrahedron.pos.x=1000;
	Tetrahedron.pos.y=-2000;
	Tetrahedron.pos.z=800;
	Tetrahedron.pos.alpha=0;
	Tetrahedron.pos.beta=0;
	Tetrahedron.pos.teta=0;
	
	err=NewShape(&Tetrahedron);
	if (err)	WayOut("\pNewShape error");
	
	err=AttachShape(&DaWorld,&Tetrahedron);		// attach shape 2 to world
	if (err)	WayOut("\pAttachShape error");
	
// CENTRAL LOOP
	
	while (!Button())
	
	{	CalcWorld(&DaWorld);			// calculate world
		CalcViewPort(&DaViewPort);		// calculate world projection in ViewPort
		RedrawViewPort(&DaViewPort);		// redraw ViewPort into image buffer
		
		// rotate Cube by 1 degree in alpha,beta and teta
		TextureCube.pos.alpha+=Deg(1);
		TextureCube.pos.beta+=Deg(1);
		TextureCube.pos.teta+=Deg(1);
	
		// rotate Tetrahedron by -1 degree in alpha and -2 in beta
		Tetrahedron.pos.alpha-=1;
		Tetrahedron.pos.beta-=2;
	}
	
	Quit();
	
}


// nada mas